##############################################
## Libraries
##############################################
library(xcms)
library(CAMERA)
library(limma)
library(MetShot)
library(gtools)
##############################################


##############################################
## Some settings
##############################################
settings=NULL
settings$general$ionmode = 1 # Ionization mode. 1 for pos and 2 for neg
settings$metshot$minint = 200 # Minimum intensity required for a feature to be interesting
settings$general$date = as.character(Sys.Date())
##############################################



##############################################
## Load your xsaFA
##############################################
load('xsaFA_pos')
load('sample_names') # Sample names we will use for selection
load('factors')      # Study sample we will use for selection
##############################################



##############################################
## Which samples do we have available?
##############################################
# Make a logical vector as long as the number of samples
available = logical(length=length(xsaFA@xcmsSet@filepaths))

# We set all to true if all samples can be used
# available[]=T

# We only want features that are found in the office plasma in this study.
available[ grep("Office-plasma", sample_names, ignore.case = TRUE) ]   =   TRUE
##############################################



##############################################
## Make peaklist
##############################################
peaklist <- getPeaklist(xsaFA, intval="maxo")
##############################################



##############################################
## Find interesting features
##############################################
# Found which features are the highest mz in a pseudo spectra (CAMERA) group. but not an isotope
max_in_group = logical(length(peaklist$mz))
for (  i in unique(peaklist$pcgroup) )  {
  if (!any((as.numeric(peaklist$pcgroup[  grepl(c("[M+H]","[M-H]")[settings$general$ionmode], peaklist[,"adduct"], fixed=TRUE)   ]) == i))) {
    masses_in_group=peaklist$mz[  peaklist$pcgroup==i   ]
    masses_in_group[    grepl('M+',peaklist[peaklist$pcgroup==i,"isotopes"],fixed=T)   ]   =  0
    group_max = max(masses_in_group)
    max_in_group[   group_max==peaklist$mz  ] = TRUE
  }
}

# Here we have a list of conditions that we later require should be fulfilled
# This would be the place to add p-values etc as another condition
cond=list()
cond[[1]] = grepl(c("[M+H]","[M-H]")[settings$general$ionmode], peaklist[,"adduct"], fixed=TRUE)  | max_in_group            # [M+H] or max mz if no [M+H]
cond[[2]] = apply(  (peaklist[,which(available)+8]>settings$metshot$minint) ,1 ,any)   #with a minimum intensity in avaliable samples

names(cond) =c(paste(c("[M+H]","[M-H]")[settings$general$ionmode]," / max mz",sep=""),"Intensity")


# Plot venn diagram for the conditions
targetgroups <- which (apply((do.call(cbind, cond)), 1, all) )
vennDiagram(     vennCounts( do.call(cbind, cond) )    )


# Prioritize by intensity
# You could of course also prioritize by p value or something else
priorities <- order(   apply(  peaklist[targetgroups,which(available)+8] ,  1, max)   )    #  by intesity

# Make report
reporttab <- groups(xsaFA@xcmsSet)[targetgroups[priorities],]
##############################################



##############################################
## Matrix showing which samples can be used for which features
##############################################
# Peaklist with only data
peaklist_dataonly = peaklist[,9:(ncol(peaklist)-3)]

# Index of the interesting features
ind_features = match(reporttab[,"mzmed"],peaklist$mz)

# features x samples matrix showing which samples can be used for which feature
# we only want to use samples that have 70% or more of the intensity of the max intensity sample
selection_mat = matrix(FALSE,nrow=length(ind_features),ncol=sum(available))

for (   i in 1:sum(available)    ){
  for (   i2 in 1:length(ind_features)    ){
    selection_mat[i2,i] = 
      peaklist_dataonly[   ind_features[i2],which(available)[i]   ] > max(peaklist_dataonly[   ind_features[i2],available   ])*0.7
  }
}


# Here we can select which sample we prefer to use. For example if we have a lot of that sample.
# Logical vector of the preferred sample(s)
available_prefer = logical(length=length(factors$person))
available_prefer[ grep("Office-plasma", sample_names, ignore.case = TRUE) ]   =   TRUE
#available_prefer[]=T # if any sample is as good as the others

# Vector showing which of the interesting features can be found in the preferred sample
in_prefer = rowSums(selection_mat[, match(which(available_prefer),which(available))])>0

# Set them to FALSE for all samples
selection_mat[in_prefer,] = FALSE

#Set them to true for preferred samples
selection_mat[in_prefer,match(which(available_prefer),which(available))] = TRUE
##############################################




##############################################
## Combine technical replicates
##
## This section uses the experimental factors to find technical replicates and combine 
## them such that we don't split features between samples that are actually the same
## In this case samples where the meal, the time and the person are the same are technical replicates.
##############################################
for ( i in 1:ncol(selection_mat)){
  for ( i2 in 1:ncol(selection_mat)){
    if (i!=i2) {
      if # if the samples are replicates
      ((factors$meal[which(available)[i]]== factors$meal[which(available)[i2]]) & 
         (factors$time[which(available)[i]] == factors$time[which(available)[i2]]) & 
         (factors$person[which(available)[i]] == factors$person[which(available)[i2]])) {
        
        selection_mat[,i2] = apply(cbind(selection_mat[,i] , selection_mat[,i2]),1,any)
        selection_mat[,i] = rep(FALSE,nrow(selection_mat)) 
      }
    }
  }
}
##############################################




##############################################
## Optimize to have least amount of experiments for each sample
##############################################
# Split into reports for each sample
reporttablist = list()
for (i in 1:ncol(selection_mat)) {
  reporttablist[[i]] = reporttab[   selection_mat[,i],  ,drop=FALSE]
}



# Generate experiments for all samples
picklists = vector("list", length(reporttablist))

for (i in 1:length(reporttablist)){
  if (nrow(reporttablist[[i]])!=0){
    picklists[[i]] <- peaklist2picklist(reporttablist[[i]], widthFactor=2, minWidth=20, fillGaps = FALSE)
  } 
}


sample_with_only_unique = numeric()   # samples with only unique masses left
counter = 0

while (   length(unique(unlist(sapply(picklists, function(x) unlist(sapply(x,function(z) z[,"mzmed"]))))))   !=  length((unlist(sapply(picklists, function(x) unlist(sapply(x,function(z) z[,"mzmed"]))))))   )   {   # continue untill number of masses equal number of unique masses.
  
  mass_freq = list()
  temp  = sapply(picklists,function(x) length(x))     # which sample has most experiments?
  temp[sample_with_only_unique]=0
  most_exp_samp = which.max(temp)
  
  
  
  for (i2 in seq(to = 1, from = length(picklists[[most_exp_samp]]), by=-1)){       # Each experiment in i'd sample, done backwards
    
    temp=  match(  unlist(sapply(picklists, function(x) unlist(sapply(x,function(z) z[,"mzmed"]))))       ,       # where does the masses in the experiment also exist?
                   picklists[[most_exp_samp]][[i2]][,"mzmed"]
    )
    
    mass_freq[[i2]] = as.numeric(table(temp))
  }
  
  
  
  if (max(sapply(mass_freq, max))==1){
    sample_with_only_unique = cbind(sample_with_only_unique, most_exp_samp)
  }else{
    
    exp_with_most_freq_mass = which.max(sapply(mass_freq, max))
    most_freq_mass = which.max(mass_freq[[exp_with_most_freq_mass]])
    
    
    # To delete!
    if (nrow(picklists[[most_exp_samp]][[exp_with_most_freq_mass]])==1){
      picklists[[most_exp_samp]][[exp_with_most_freq_mass]] = NULL
    }else{
      picklists[[most_exp_samp]][[exp_with_most_freq_mass]] =  picklists[[most_exp_samp]][[exp_with_most_freq_mass]][-most_freq_mass,]
    }
    
    
    
  }
  counter = counter+1
}
##############################################





##############################################
## Create methods for waters
##############################################
sample_list = character()
picklists_final = vector("list", length(reporttablist))
methodnames = vector("list", length(reporttablist))

# Select which collision energies to create experiments for
collisionEnergy <- c(10,20,30)

# Here you need to provide a sample experiment file setup correctly for your instrument
templateFile    <- c('neg_30eV_30eV.EXP')


# Now we generate the experiment files and make an excel file with a sample list
for (i in 1:length(picklists)){
  
  if (!all(picklists[[i]]=='NA')){
    picklists[[i]][   picklists[[i]]=='NA'  ] = NULL
    
    
    if (length(picklists[[i]])!=0){
      methodname <- paste(getwd(),'/',settings$general$date,'/experiments_',c('pos','neg')[settings$general$ionmode],'/',i,'_',(strsplit(gsub(" *$", "", sample_names[which(available)][i]), ','))[[1]][1],"_",factors$day[which(available)[i]],"_",factors$time[which(available)[i]],'_',c('pos','neg')[settings$general$ionmode], sep="")
      methodnames[[i]]=methodname
      
      temp_pick = peaklist2picklist(do.call(rbind,picklists[[i]]),widthFactor=1, minWidth=20)
      picklists_final[[i]] <- temp_pick
      
      picklists2waters(temp_pick,
                       methodPrefix=methodname,
                       template=templateFile,
                       TOFConeVoltage=30,
                       MSMSManual_ListCollisionEnergy=collisionEnergy,
                       lockMassMZ=c(556.2771,554.2615)[settings$general$ionmode],
                       MSmode=c("positive","negative")[settings$general$ionmode])
      
      
      
      File_Name = paste(rep(settings$general$date,length(temp_pick)*length(collisionEnergy)), sprintf("%03d",1:(length(temp_pick)*length(collisionEnergy)))    ,sep="-")
            
      
      MS_File = paste(rep(paste(rep(basename(methodname),length(temp_pick)),1:length(temp_pick),sep="_"),length(collisionEnergy)),'-',rep(collisionEnergy,1,each=length(temp_pick)),'eV',sep="")
      order = mixedorder(gsub('-','_',MS_File))
      File_Text=MS_File = MS_File[order]
      Inlet_file = rep('mkri6min04flow_10uL',length(temp_pick)*length(collisionEnergy))
      
      sample_list = rbind(sample_list,cbind(File_Name,File_Text,MS_File,Inlet_file))
      
      
      
    } 
  }
}

colnames(sample_list)=c('File Name','File Text','MS File','Inlet_file')

write.csv(sample_list,
          paste(getwd(),'/',settings$general$date,'/experiments_',c('pos','neg')[settings$general$ionmode],'/samplelist.csv',sep="")
)
##############################################